"""delete "script" from the table "run"

Revision ID: f9a742bb2297
Revises: cafceacada62
Create Date: 2024-01-30 13:48:55.138737

"""
import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = 'f9a742bb2297'
down_revision = 'cafceacada62'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute('PRAGMA foreign_keys=OFF;')
    with op.batch_alter_table('run', schema=None) as batch_op:
        batch_op.drop_column('script_old')
    op.execute('PRAGMA foreign_keys=ON;')

    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.execute('PRAGMA foreign_keys=OFF;')
    with op.batch_alter_table('run', schema=None) as batch_op:
        batch_op.add_column(sa.Column('script_old', sa.TEXT(), nullable=True))
    op.execute('PRAGMA foreign_keys=ON;')

    # ### end Alembic commands ###
