query HistoryRuns(
  $before: String
  $after: String
  $first: Int
  $last: Int
) {
  rdb {
    runs(before: $before, after: $after, first: $first, last: $last) {
      pageInfo {
        startCursor
        endCursor
        hasNextPage
        hasPreviousPage
      }
      totalCount
      edges {
        cursor
        node {
          id
          runNo
        }
      }
    }
  }
}
